remove STL vector
[c11tester.git] / main.cc
diff --git a/main.cc b/main.cc
index 9375ac0d80e0a86aa72014d75972b712704372b2..5396823334df9a0b47f3077f3c352cc6774e7e1b 100644 (file)
--- a/main.cc
+++ b/main.cc
 #include "snapshot-interface.h"
 #include "plugins.h"
 
-static void param_defaults(struct model_params *params)
+void param_defaults(struct model_params *params)
 {
        params->verbose = !!DBG_ENABLED();
        params->uninitvalue = 0;
        params->maxexecutions = 10;
+       params->nofork = false;
 }
 
 static void print_usage(const char *program_name, struct model_params *params)
@@ -54,6 +55,7 @@ static void print_usage(const char *program_name, struct model_params *params)
                "-x, --maxexec=NUM           Maximum number of executions.\n"
                "                            Default: %u\n"
                "                            -o help for a list of options\n"
+               "-n                          No fork\n"
                " --                         Program arguments follow.\n\n",
                program_name,
                params->verbose,
@@ -84,7 +86,7 @@ bool install_plugin(char * name) {
 
 static void parse_options(struct model_params *params, int argc, char **argv)
 {
-       const char *shortopts = "ht:o:u:x:v::";
+       const char *shortopts = "hnt:o:u:x:v::";
        const struct option longopts[] = {
                {"help", no_argument, NULL, 'h'},
                {"verbose", optional_argument, NULL, 'v'},
@@ -92,7 +94,7 @@ static void parse_options(struct model_params *params, int argc, char **argv)
                {"analysis", required_argument, NULL, 't'},
                {"options", required_argument, NULL, 'o'},
                {"maxexecutions", required_argument, NULL, 'x'},
-               {0, 0, 0, 0}                                            /* Terminator */
+               {0, 0, 0, 0}    /* Terminator */
        };
        int opt, longindex;
        bool error = false;
@@ -101,6 +103,9 @@ static void parse_options(struct model_params *params, int argc, char **argv)
                case 'h':
                        print_usage(argv[0], params);
                        break;
+               case 'n':
+                       params->nofork = true;
+                       break;
                case 'x':
                        params->maxexecutions = atoi(optarg);
                        break;
@@ -121,7 +126,7 @@ static void parse_options(struct model_params *params, int argc, char **argv)
                                error = true;
                }
                break;
-               default:                                                /* '?' */
+               default:        /* '?' */
                        error = true;
                        break;
                }
@@ -156,33 +161,6 @@ static void install_trace_analyses(ModelExecution *execution)
        }
 }
 
-/** The model_main function contains the main model checking loop. */
-static void model_main()
-{
-       struct model_params params;
-
-       param_defaults(&params);
-       register_plugins();
-
-       parse_options(&params, main_argc, main_argv);
-
-       //Initialize race detector
-       initRaceDetector();
-
-       snapshot_stack_init();
-
-       if (!model)
-               model = new ModelChecker();
-       model->setParams(params);
-       install_trace_analyses(model->get_execution());
-
-       snapshot_record(0);
-       model->run();
-       delete model;
-
-       DEBUG("Exiting\n");
-}
-
 /**
  * Main function.  Just initializes snapshotting library and the
  * snapshotting library calls the model_main function.
@@ -193,20 +171,36 @@ int main(int argc, char **argv)
        main_argv = argv;
 
        /*
* If this printf statement is removed, CDSChecker will fail on an
         * assert on some versions of glibc.  The first time printf is
         * called, it allocated internal buffers.  We can't easily snapshot
         * libc since we also use it.
         */
 
-       printf("CDSChecker\n"
-                                "Copyright (c) 2013 Regents of the University of California. All rights reserved.\n"
+       printf("C11Tester\n"
+                                "Copyright (c) 2013 and 2019 Regents of the University of California. All rights reserved.\n"
                                 "Distributed under the GPLv2\n"
-                                "Written by Brian Norris and Brian Demsky\n\n");
+                                "Written by Weiyu Luo, Brian Norris, and Brian Demsky\n\n");
 
        /* Configure output redirection for the model-checker */
        redirect_output();
 
-       /* Let's jump in quickly and start running stuff */
-       snapshot_system_init(10000, 1024, 1024, 4000, &model_main);
+       //Initialize snapshotting library and model checker object
+       if (!model) {
+               snapshot_system_init(10000, 1024, 1024, 40000);
+               model = new ModelChecker();
+               model->startChecker();
+       }
+
+       register_plugins();
+
+       //Parse command line options
+       model_params *params = model->getParams();
+       parse_options(params, main_argc, main_argv);
+
+
+       install_trace_analyses(model->get_execution());
+
+       model->startMainThread();
+       DEBUG("Exiting\n");
 }
@@ -193,20 +171,36 @@ int main(int argc, char **argv)
 
        /*
-        * If this printf statement is removed, CDSChecker will fail on an
+        * If this printf statement is removed, C11Tester will fail on an
         * assert on some versions of glibc.  The first time printf is
         * called, it allocated internal buffers.  We can't easily snapshot